@@ -3,6 +3,7 @@
 """
 
 from numpy import ndarray, zeros, concatenate, linspace, amin, amax
+import numpy as np
 import matplotlib.pyplot as plt
 
 from boututils.datafile import DataFile
@@ -211,6 +212,8 @@ def gridcontourf(
     xlabel="Major radius [m]",
     ylabel="Height [m]",
     separatrix=False,
+    log=False,
+    remove_xguards=False,
 ):
     """Plots a 2D contour plot, taking into account branch cuts
     (X-points).
@@ -240,6 +243,10 @@ def gridcontourf(
         Labels for the x/y axes
     separatrix : bool, optional
         Add separatrix
+    log : bool, optional
+        Use a logarithmic scale for the color
+    remove_xguards : bool, optional
+        Convert two guard cells into one value on the boundary
 
     Returns
     -------
@@ -309,6 +316,21 @@ def gridcontourf(
     if data2d.shape != (nx, ny):
         raise ValueError("Dimensions do not match")
 
+    if remove_xguards:
+
+        def remove_guards(arr2d):
+            result = arr2d[1:-1, :].copy()
+            result[0, :] = 0.5 * (result[0, :] + result[1, :])
+            result[-1, :] = 0.5 * (result[-1, :] + result[-2, :])
+            return result
+
+        R = remove_guards(R)
+        Z = remove_guards(Z)
+        data2d = remove_guards(data2d)
+        # Adjust x indices for the separatrices
+        ix1 -= 1
+        ix2 -= 1
+
     add_colorbar = False
     if ax is None:
         fig = plt.figure()
@@ -325,7 +347,14 @@ def gridcontourf(
         maxd = max([maxd, abs(mind)])
         mind = -maxd
 
-    levels = linspace(mind, maxd, nlevel, endpoint=True)
+    if log:
+        levels = np.logspace(np.log10(mind), np.log10(maxd), nlevel, endpoint=True)
+        from matplotlib import colors
+
+        norm = colors.LogNorm(vmin=mind, vmax=maxd)
+    else:
+        levels = linspace(mind, maxd, nlevel, endpoint=True)
+        norm = None
 
     ystart = 0  # Y index to start the next section
     if j11 >= 0:
@@ -336,6 +365,7 @@ def gridcontourf(
             data2d[:, ystart : (j11 + 1)],
             levels,
             cmap=cmap,
+            norm=norm,
         )
 
         yind = [j11, j22 + 1]
@@ -345,6 +375,7 @@ def gridcontourf(
             data2d[:ix1, yind].transpose(),
             levels,
             cmap=cmap,
+            norm=norm,
         )
 
         ax.contourf(
@@ -353,6 +384,7 @@ def gridcontourf(
             data2d[ix1:, j11 : (j11 + 2)],
             levels,
             cmap=cmap,
+            norm=norm,
         )
         ystart = j11 + 1
 
@@ -363,6 +395,7 @@ def gridcontourf(
             data2d[:ix1, yind].transpose(),
             levels,
             cmap=cmap,
+            norm=norm,
         )
 
     # Inner SOL
@@ -372,6 +405,7 @@ def gridcontourf(
         data2d[:, ystart : (j21 + 1)],
         levels,
         cmap=cmap,
+        norm=norm,
     )
     ystart = j21 + 1
 
@@ -385,9 +419,15 @@ def gridcontourf(
             data2d[ix1:, j21 : (j21 + 2)],
             levels,
             cmap=cmap,
+            norm=norm,
         )
         ax.contourf(
-            R[:, ystart:nin], Z[:, ystart:nin], data2d[:, ystart:nin], levels, cmap=cmap
+            R[:, ystart:nin],
+            Z[:, ystart:nin],
+            data2d[:, ystart:nin],
+            levels,
+            cmap=cmap,
+            norm=norm,
         )
 
         # Outer leg
@@ -397,6 +437,7 @@ def gridcontourf(
             data2d[:, nin : (j12 + 1)],
             levels,
             cmap=cmap,
+            norm=norm,
         )
         ax.contourf(
             R[ix1:, j12 : (j12 + 2)],
@@ -404,6 +445,7 @@ def gridcontourf(
             data2d[ix1:, j12 : (j12 + 2)],
             levels,
             cmap=cmap,
+            norm=norm,
         )
         ystart = j12 + 1
 
@@ -414,6 +456,7 @@ def gridcontourf(
             data2d[:ix1, yind].transpose(),
             levels,
             cmap=cmap,
+            norm=norm,
         )
 
         yind = [j21 + 1, j12]
@@ -423,6 +466,7 @@ def gridcontourf(
             data2d[:ix1, yind].transpose(),
             levels,
             cmap=cmap,
+            norm=norm,
         )
     else:
         ystart -= 1
@@ -433,6 +477,7 @@ def gridcontourf(
         data2d[:, ystart : (j22 + 1)],
         levels,
         cmap=cmap,
+        norm=norm,
     )
 
     ystart = j22 + 1
@@ -445,9 +490,15 @@ def gridcontourf(
             data2d[ix1:, j22 : (j22 + 2)],
             levels,
             cmap=cmap,
+            norm=norm,
         )
         ax.contourf(
-            R[:, ystart:ny], Z[:, ystart:ny], data2d[:, ystart:ny], levels, cmap=cmap
+            R[:, ystart:ny],
+            Z[:, ystart:ny],
+            data2d[:, ystart:ny],
+            levels,
+            cmap=cmap,
+            norm=norm,
         )
 
         # X-point
@@ -474,7 +525,14 @@ def gridcontourf(
                 data2d[ix1 - 1, j22],
             ],
         ]
-        ax.contourf(Rx, Zx, Dx, levels, cmap=cmap)
+        ax.contourf(
+            Rx,
+            Zx,
+            Dx,
+            levels,
+            cmap=cmap,
+            norm=norm,
+        )
 
     if add_colorbar:
         fig.colorbar(con)
